%% EIS_filter.m is a function built for the course 
%% CCIT4076 Engineering and Information Science 
%% HKU SPACE Community College (Fall 2022)

%% author: W. Y. Keung
%% last update: Sept 13, 2022
%% all rights reserved

function y = EIS_Filter(x, fs, fl, fh)
    % Dealing with improper cases here
    fh = min(fs/2, fh); 
    fl = max(1e-9, fl); 
    
    if exist('OCTAVE_VERSION', 'builtin')
        pkg load signal;
        pkg load communications;
    end
    
    if fl > fh
      nvm = fh; fh = fl; fl = fh; 
      clear nvm
    end

    if (abs(fh - fl) < 1) 
      printf("The passband is too small! Retry with a wider bandwidth!\n");
      y = NaN;
      return;
    elseif (abs(fh - fl) > fs/2)
      printf("The passband is too large! Retry with a narrower bandwidth!\n");
      y = NaN;
      return;
    end
    
    % Main duty 
    % pkg load signal;
    if fh == inf
      fh = fs/2;
      b = fir1(ceil(0.01*length(x)), fl./(fs/2), 'high');   
      flim = fh;
    else 
      b = fir1(ceil(0.01*length(x)), [fl./(fs/2), fh./(fs/2)]);
      flim = 1.8*fh;
    end
     
    y = filter(b, 1, x);
    [h, w] = freqz(b, 1, length(x));
    

    figure(); 
    subplot(3, 1, 1); plot(fs*w./(pi), abs(fft(x))./sqrt(length(x)), 'b'); hold on; grid on; grid minor; 
    xlim([0 flim]); ylim([0, 1.1*max(abs(fft(x))./sqrt(length(x)))]);
    line([fl fl], [0 1.5*max(abs(fft(x))./sqrt(length(x)))], 'color', 'r'); line([fh fh], [0 1.5*max(abs(fft(x))./sqrt(length(x)))], 'color', 'r');
    xlabel('f/Hz'); ylabel('Input Signal Spectrum')

    subplot(3, 1, 2); 
    plot(fs*w./(2*pi), abs(h), 'g-o', 'linewidth', 1,'markersize', 0.1); hold on; grid on; grid minor; 
    xlim([0 flim]); ylim([0, 1.5*max(abs(h))]);
    line([fl fl], [0 10], 'color', 'r'); line([fh fh], [0 1.5*max(abs(fft(x))./sqrt(length(x)))], 'color', 'r'); 
    xlabel('f/Hz');  ylabel('Filter Response')
    subplot(3, 1, 3); 
    plot(fs*w./(pi), abs(fft(y))./sqrt(length(y)), 'color', [0.3 0 0.3]); hold on; grid on; grid minor; 
    xlim([0 flim]); ylim([0, 1.1*max(abs(fft(x))./sqrt(length(x)))]);
    line([fl fl], [0 1.5*max(abs(fft(x))./sqrt(length(x)))], 'color', 'r'); line([fh fh], [0 1.5*max(abs(fft(x))./sqrt(length(x)))], 'color', 'r'); 
    xlabel('f/Hz'); ylabel('Output Signal Spectrum')
    
  
end